@@ -9,11 +9,11 @@ import click
                    '\b\n 1D scan: "poi_name=<poi_min>_<poi_max>_<step>"'
                    '\b\n 2D scan: "poi_1_name=<poi_1_min>_<poi_1_max>_<step_1>,'
                    '\b\n           poi_2_name=<poi_2_min>_<poi_2_max>_<step_2>"')
-@click.option('-f', '--filter', 'filter_expr', default=None, show_default=True,
+@click.option('--filter', 'filter_expr', default=None, show_default=True,
               help='\b Filter parameter points by expression.\n'
                    '\b Example: "mass=2*,350,400,450;klambda=1.*,2.*,-1.*,-2.*"\n'
                    '\b Refer to documentation for more information\n')
-@click.option('-e', '--exclude', 'exclude_expr', default=None, show_default=True,
+@click.option('--exclude', 'exclude_expr', default=None, show_default=True,
               help='\b Exclude parameter points by expression.\n'
                    '\b Example: "mass=2*,350,400,450;klambda=1.*,2.*,-1.*,-2.*"\n'
                    '\b Refer to documentation for more information\n')
@@ -57,6 +57,8 @@ import click
               help='Binned likelihood')
 @click.option('-e', '--eps', type=float, default=1.0, show_default=True,
               help='Convergence criterium')
+@click.option('--retry', type=int, default=0, show_default=True,
+              help='Maximum number of retries upon a failed fit')
 @click.option('--strategy', type=int, default=1, show_default=True,
               help='Default minimization strategy')
 @click.option('--print_level', type=int, default=-1, show_default=True,
