@@ -1,6 +1,7 @@
 from typing import Optional, Union, List
 
 from quickstats.components import AbstractObject, ExtendedModel, ExtendedMinimizer
+from quickstats.components.basics import WSVariables
 from quickstats.utils.common_utils import get_class_that_defined_method
 import ROOT
 
@@ -14,7 +15,8 @@ class AnalysisObject(AbstractObject):
                  num_cpu:int=1, offset:bool=True, optimize:int=2, eigen:bool=False,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
                  max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
-                 int_bin_precision:float=-1., verbosity:Optional[Union[int, str]]="INFO"):
+                 int_bin_precision:float=-1., preset_param:bool=False,
+                 verbosity:Optional[Union[int, str]]="INFO"):
         super().__init__(verbosity=verbosity)    
         self.model = None
         self.minimizer = None
@@ -25,11 +27,31 @@ class AnalysisObject(AbstractObject):
                              snapshot_name, fix_cache, fix_multi, verbosity=verbosity)
             self.save_snapshot("initialSnapshot")
             self.set_poi(poi_name)
+            if preset_param:
+                self.preset_parameters()
             self.setup_parameters(fix_param, profile_param, update_snapshot=True)
             self.setup_minimizer(minimizer_type, minimizer_algo, precision, eps, retry, strategy,
                                  num_cpu, offset, optimize, eigen, max_calls, max_iters, print_level,
                                  timer, constrain_nuis, batch_mode, int_bin_precision,
                                  verbosity=verbosity)
+            self.sanity_check()
+        
+    def sanity_check(self):
+        aux_vars = self.model.get_variables("auxiliary")
+        floating_aux_vars = [v.GetName() for v in aux_vars if not v.isConstant()]
+        if len(floating_aux_vars) > 0:
+            self.stdout.warning("WARNING: The following auxiliary variables (variables that are not "
+                                "part of the POIs, observables, nuisance parameters and global "
+                                f"observables) are floating: {','.join(floating_aux_vars)}. If this is "
+                                "not intended, please make sure to fix them before fitting.", "red")            
+    
+    def preset_parameters(self):
+        ROOT.RooStats.SetAllConstant(self.model.global_observables, True)
+        ROOT.RooStats.SetAllConstant(self.model.nuisance_parameters, False)
+        ROOT.RooStats.SetAllConstant(self.model.pois, True)
+        self.stdout.info("INFO: Preset POIs as constant parameters.")
+        self.stdout.info("INFO: Preset nuisance parameters as floating parameters.")
+        self.stdout.info("INFO: Preset global observables as constant parameters.")
     
     @property
     def use_blind_range(self):
@@ -104,19 +126,20 @@ class AnalysisObject(AbstractObject):
         
         minimizer = ExtendedMinimizer("Minimizer", self.model.pdf, self.model.data, verbosity=verbosity)
         
-        nll_commands = [ROOT.RooFit.NumCPU(num_cpu, 3), 
-                        ROOT.RooFit.Offset(offset)]
-        if (constrain_nuis and self.model.nuisance_parameters):
-            #nll_commands.append(ROOT.RooFit.Constrain(self.model.nuisance_parameters))
-            nll_commands.append(ROOT.RooFit.GlobalObservables(self.model.global_observables))
+        nll_options = {
+            "num_cpu": num_cpu,
+            "offset": offset,
+            "batch_mode": batch_mode,
+            "int_bin_precision": int_bin_precision
+        }
+        if constrain_nuis:
+            nll_options['constrain'] = self.model.nuisance_parameters
+            nll_options['global_observables'] = self.model.global_observables
             conditional_obs = self.model.model_config.GetConditionalObservables()
             if conditional_obs:
-                nll_commands.append(ROOT.RooFit.ConditionalObservables(conditional_obs))
+                nll_options['conditional_observables'] = conditional_obs
             #nll_commands.append(ROOT.RooFit.ExternalConstraints(ROOT.RooArgSet()))
-        if (hasattr(ROOT.RooFit, "BatchMode")):
-            nll_commands.append(ROOT.RooFit.BatchMode(batch_mode))
-        if (hasattr(ROOT.RooFit, "IntegrateBins")):
-            nll_commands.append(ROOT.RooFit.IntegrateBins(int_bin_precision))
+        
         minimizer_options = {
             'minimizer_type'   : minimizer_type,
             'minimizer_algo'   : minimizer_algo,
@@ -132,9 +155,11 @@ class AnalysisObject(AbstractObject):
             'timer'            : timer
         }
 
-        minimizer.configure(nll_commands=nll_commands, **minimizer_options)
-        
+        minimizer.configure(**minimizer_options)
+        minimizer.configure_nll(**nll_options)
         self.minimizer = minimizer
+        self.default_nll_options = nll_options
+        self.default_minimizer_options = minimizer_options
     
     def set_data(self, data_name:str='combData'):
         data = self.model.workspace.data(data_name)
@@ -165,27 +190,16 @@ class AnalysisObject(AbstractObject):
                 self.model.workspace.loadSnapshot(snapshot_name)
                 self.stdout.debug(f"DEBUG: Loaded snapshot \"{snapshot_name}\"")
     
-    def get_variables(self, name:str):
-        if name == "nuis":
-            variables = self.model.nuisance_parameters
-        elif name == "globs":
-            variables = self.model.global_observables
-        elif name == "core":
-            variables = ROOT.RooArgSet()
-            variables.add(self.model.nuisance_parameters)
-            variables.add(self.model.global_observables)
-            variables.add(self.model.pois)
-        else:
-            raise RuntimeError("unknown name for variable set")
-        return variables
+    def get_variables(self, variable_type:Union[str, WSVariables], sort:bool=True):
+        return self.model.get_variables(variable_type, sort=sort)
     
     def save_snapshot(self, snapshot_name:Optional[str]=None, 
-                      variables:Optional[Union[ROOT.RooArgSet, str]]=None):
+                      variables:Optional[Union[ROOT.RooArgSet, str, WSVariables]]=None):
         if snapshot_name is not None:
             if variables is None:
                 self.model.workspace.saveSnapshot(snapshot_name, self.model.workspace.allVars())
             else:
-                if isinstance(variables, str):
+                if isinstance(variables, (str, WSVariables)):
                     variables = self.get_variables(variables)
                 self.model.workspace.saveSnapshot(snapshot_name, variables)
             self.stdout.debug(f"DEBUG: Saved snapshot \"{snapshot_name}\"")
