@@ -11,6 +11,7 @@ from quickstats.utils.root_utils import load_macro
 from quickstats.utils import common_utils
 from quickstats.utils.io import Verbosity
 from quickstats.maths.numerics import pretty_value
+from quickstats.components.basics import WSVariables
 
 import ROOT
 
@@ -26,9 +27,10 @@ class AsymptoticCLs(AnalysisObject):
         'inj': 'Injected'
     }
     
-    def __init__(self, filename:str, poi_name:str=None, data_name:str='combData', 
-                 CL:float=0.95, precision:float=0.005, do_tilde:bool=True,
-                 do_better_bands:bool=True, predictive_fit:bool=False,
+    def __init__(self, filename:str, poi_name:str=None, data_name:str='combData',
+                 asimov_data_name:Optional[str]=None,
+                 CL:float=0.95, precision:float=0.005, adjust_fit_range:bool=True,
+                 do_tilde:bool=True, do_better_bands:bool=True, predictive_fit:bool=False,
                  better_negative_bands:bool=False, do_blind:bool=True,
                  binned_likelihood:bool=True, fix_param:str='', profile_param:str='',
                  ws_name:Optional[str]=None, mc_name:Optional[str]=None,
@@ -74,6 +76,7 @@ class AsymptoticCLs(AnalysisObject):
             'constrain_nuis': constrain_nuis,
             'batch_mode': batch_mode,
             'int_bin_precision': int_bin_precision,
+            'preset_param': True,
             'verbosity': verbosity
         }
         super().__init__(**config)
@@ -84,9 +87,25 @@ class AsymptoticCLs(AnalysisObject):
         self.use_predictive_fit = predictive_fit
         self.do_better_bands = do_better_bands
         self.better_negative_bands = better_negative_bands
+        self.adjust_fit_range = adjust_fit_range
         self.direction = 1
         self.global_status = 0
         
+        # define asimov data
+        if asimov_data_name is not None:
+            asimov_data_0 = self.model.workspace.data(asimov_data_name)
+            if not asimov_data_0:
+                raise RuntimeError(f'failed to load dataset \"{asimov_data_name}\"')
+            self.asimov_data_0 = asimov_data_0
+            glob_snap_name = self.get_conditional_snapsot_name(0, "global_observables")
+            nuis_snap_name = self.get_conditional_snapsot_name(0, "nuisance_parameters")
+            if (not self.model.workspace.loadSnapshot(glob_snap_name)) or \
+            (not self.model.workspace.loadSnapshot(nuis_snap_name)):
+                raise RuntimeError("when using user-defined background-only Asimov data, "
+                                   f"snapshots {nuis_snap_name} and {glob_snap_name} must be defined.")
+        else:
+            self.asimov_data_0 = None
+        
         # define simplified names
         self.ws = self.model.workspace
         self.globs = self.model.global_observables
@@ -129,22 +148,6 @@ class AsymptoticCLs(AnalysisObject):
         if ( val < 0 ) and (self.poi.getMin() > val):
             self.poi.setMin(2*val)
         self.poi.setVal(val)
-    
-    """
-    # superceded by AnalysisObject
-    def save_snapshot(self, name:str, target:str="nuis"):
-        if target == "nuis":
-            self.ws.saveSnapshot(name, self.nuis)
-        elif target == "globs":
-            self.ws.saveSnapshot(name, self.globs)
-            
-    def load_snapshot(self, name:str):
-        snapshot = self.ws.getSnapshot(name)
-        if snapshot == None:
-            raise RuntimeError('snapshot "{}" does not exist'.format(name))
-        self.stdout.debug('INFO: Loaded snapshot "{}"'.format(name))
-        self.ws.loadSnapshot(name)
-    """
             
     def save_nll_snapshot(self, nll, mu):
         name = self.nll_snapshot_name.format(nll_name=nll.GetName(), mu=pretty_value(mu))
@@ -154,17 +157,20 @@ class AsymptoticCLs(AnalysisObject):
         name = self.nll_snapshot_name.format(nll_name=nll.GetName(), mu=pretty_value(mu))
         self.load_snapshot(name) 
     
-    def load_conditional_snapshot(self, mu:float, target:str="nuis"):
+    def load_conditional_snapshot(self, mu:float, 
+                                  target:Union[str, WSVariables]=WSVariables.NUISANCE_PARAMETERS):
         name = self.get_conditional_snapsot_name(mu, target)
         self.load_snapshot(name)
     
-    def get_conditional_snapsot_name(self, mu:float, target:str="nuis"):
-        if target == "nuis":
+    def get_conditional_snapsot_name(self, mu:float, 
+                                     target:Union[str, WSVariables]=WSVariables.NUISANCE_PARAMETERS):
+        target = WSVariables.parse(target)
+        if target == WSVariables.NUISANCE_PARAMETERS:
             name = self.model._DEFAULT_NAMES_['conditional_nuis'].format(mu=pretty_value(mu))
-        elif target == "globs":
+        elif target == WSVariables.GLOBAL_OBSERVABLES:
             name = self.model._DEFAULT_NAMES_['conditional_globs'].format(mu=pretty_value(mu))
         else:
-            raise ValueError('unknown conditional snapshot target "{}"'.format(target))
+            raise ValueError(f'unknown conditional snapshot target "{target}"')
         return name
 
     def get_nll_val(self, nll:"ROOT.RooNLLVar"):
@@ -175,7 +181,7 @@ class AsymptoticCLs(AnalysisObject):
             raise RuntimeError('Failed to load snapshot for nll "{}"'.format(nll.GetName()))
         if Verbosity.DEBUG >= self.stdout.verbosity:
             self.stdout.debug("DEBUG: Before Fit ----------------------------------------------")
-            self.model.print_summary(items=['poi', 'nuisance_parameter', 'global_observable'])             
+            self.model.print_summary(items=['poi', 'nuisance_parameter', 'global_observable'])
         self.global_status += self.minimizer.minimize(nll)
         self.minimizer_calls += 1
         nll_val = nll.getVal()
@@ -400,7 +406,7 @@ class AsymptoticCLs(AnalysisObject):
         
     def fill_mappings(self, nll:"ROOT.RooNLLVar", dataset:"ROOT.RooDataSet",
                       mu_hat:float, nll_mu_hat:float):
-        snapshot_name = self.get_conditional_snapsot_name(mu_hat, target="globs")
+        snapshot_name = self.get_conditional_snapsot_name(mu_hat, target=WSVariables.GLOBAL_OBSERVABLES)
         self.nll_maps[nll] = {}
         self.nll_maps[nll]['dataset'] = dataset
         self.nll_maps[nll]['snapshot'] = snapshot_name
@@ -465,10 +471,12 @@ class AsymptoticCLs(AnalysisObject):
         self.load_snapshot(snapshot_0_name)
         asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma,
                                                    do_fit=False, modify_globs=True,
-                                                   restore_states=1)
+                                                   restore_states=1,
+                                                   minimizer_options=self.default_minimizer_options,
+                                                   nll_options=self.default_nll_options)
         asimov_n_nll = self.create_nll(asimov_data_n)
-        self.load_conditional_snapshot(n_times_sigma, target="globs")
-        self.load_conditional_snapshot(n_times_sigma, target="nuis")
+        self.load_conditional_snapshot(n_times_sigma, target=WSVariables.GLOBAL_OBSERVABLES)
+        self.load_conditional_snapshot(n_times_sigma, target=WSVariables.NUISANCE_PARAMETERS)
         self.set_poi_value(n_times_sigma)
         nll_value = asimov_n_nll.getVal()
         self.save_nll_snapshot(asimov_n_nll, n_times_sigma)
@@ -491,21 +499,29 @@ class AsymptoticCLs(AnalysisObject):
         obs_nll = self.create_nll(self.data)
         self.nll_maps[obs_nll] = {"dataset": self.data, "snapshot": self.nom_glob_name}
         self.dataset_maps[self.data] = obs_nll
-        self.save_snapshot(self.nom_glob_name, variables="globs")
-        self.save_snapshot(self.nom_nuis_name, variables="nuis")
+        self.save_snapshot(self.nom_glob_name, variables=WSVariables.GLOBAL_OBSERVABLES)
+        self.save_snapshot(self.nom_nuis_name, variables=WSVariables.NUISANCE_PARAMETERS)
         
-        asimov_data_0 = self.model.generate_asimov(self.poi.GetName(), poi_val=0,
-                                                   poi_profile=0, 
-                                                   do_fit=not self.do_blind,
-                                                   modify_globs=not self.do_blind,
-                                                   dataset=self.data,
-                                                   restore_states=1)
+        if self.asimov_data_0 is None:
+            # generate asimov dataset on the fly
+            asimov_data_0 = self.model.generate_asimov(self.poi.GetName(), poi_val=0,
+                                                       poi_profile=0, 
+                                                       do_fit=not self.do_blind,
+                                                       modify_globs=not self.do_blind,
+                                                       dataset=self.data,
+                                                       restore_states=1,
+                                                       minimizer_options=self.default_minimizer_options,
+                                                       nll_options=self.default_nll_options)
+        else:
+            # use cutom asimov dataset
+            asimov_data_0 = self.asimov_data_0
+            
         asimov_0_nll = self.create_nll(asimov_data_0)
         self.summary[0] = {'dataset': asimov_data_0, 'nll': asimov_0_nll}
         self.set_poi_value(0)
         self.save_nll_snapshot(asimov_0_nll, 0)
-        self.load_conditional_snapshot(0, target="globs")
-        self.load_conditional_snapshot(0, target="nuis")
+        self.load_conditional_snapshot(0, target=WSVariables.GLOBAL_OBSERVABLES)
+        self.load_conditional_snapshot(0, target=WSVariables.NUISANCE_PARAMETERS)
         nll_value = asimov_0_nll.getVal()
         self.fill_mappings(asimov_0_nll, asimov_data_0, 0, nll_value)
         
@@ -520,6 +536,7 @@ class AsymptoticCLs(AnalysisObject):
         # restrict poi range for computing sigma bands
         self.poi.setRange(-5*sigma, 5*sigma)
         
+        
         # compute better sigma bands
         bands_to_improve = []
         if self.do_better_bands:
@@ -532,9 +549,11 @@ class AsymptoticCLs(AnalysisObject):
             limits[n] = self.evaluate_limit_band(n, med_limit)
         
         # compute observed limit
-        self.load_conditional_snapshot(0, target="nuis")
+        self.load_conditional_snapshot(0, target=WSVariables.NUISANCE_PARAMETERS)
         # relax the range for observed limit
-        self.poi.setRange(poi_range_tmp[0], poi_range_tmp[1])
+        if not self.adjust_fit_range:
+            self.poi.setRange(poi_range_tmp[0], poi_range_tmp[1])
+
         obs_limit = 0 if self.do_blind else self.get_limit(obs_nll, med_limit, 'obs')
         limits['obs'] = obs_limit
         
@@ -546,5 +565,5 @@ class AsymptoticCLs(AnalysisObject):
         
         self.print_summary()
         
-        self.stdout.info('INFO: Finished with {} calls to minimize(nll)'.format(self.minimizer_calls))
+        self.stdout.info(f'INFO: Finished with {self.minimizer_calls} calls to minimize(nll)')
         return limits
\ No newline at end of file
