@@ -409,7 +409,7 @@ class ExtendedModel(AbstractObject):
             self.stdout.info(f"INFO: Set parameter \"{name}\" at value {param.getVal()} "
                              f"[{param.getMin()}, {param.getMax()}] {state_str}")
 
-    def set_parameter_defaults(source:"ROOT.RooArgSet", value:float=None, error:float=None,
+    def set_parameter_defaults(self, source:"ROOT.RooArgSet", value:float=None, error:float=None,
                                constant:bool=None, remove_range:bool=None, target:List[str]=None):
         for param in source:
             if (not target) or (param.GetName() in target):
