@@ -65,7 +65,8 @@ class ParameterisedAsymptoticCLs(ParameterisedRunner):
             if outname is not None:
                 asymptotic_cls.save(outname, summary=save_summary)
             return asymptotic_cls.limits
-        except:
+        except Exception as e:
+            sys.stdout.write(f"{e}\n")
             return {}
         
     def get_cache_dir(self):
