@@ -1,4 +1,4 @@
-from typing import Optional, Union, Dict, List, Tuple
+from typing import Optional, Union, Dict, List, Tuple, Callable
 from cycler import cycler
 
 import matplotlib
@@ -54,6 +54,11 @@ class AbstractPlot:
             self.prop_cycle = (cycler(color=self.COLOR_CYCLE))
         else:
             self.prop_cycle = (cycler(color=color_cycle))
+            
+        self.annotation_list = []
+        
+    def add_annotation(self, **kwargs):
+        self.annotation_list.append(kwargs)
         
     def get_default_legend_order(self):
         return []
@@ -103,6 +108,12 @@ class AbstractPlot:
                           analysis_label_options=self.analysis_label_options,
                           figure_index=self.figure_index,
                           **kwargs)
+        for annotation_kwargs in self.annotation_list:
+            _annotation_kwargs = combine_dict(self.styles['annotation'], annotation_kwargs)
+            if isinstance(ax, tuple):
+                ax[0].annotate(**_annotation_kwargs)
+            else:
+                ax.annotate(**_annotation_kwargs)
         self.figure = matplotlib.pyplot.gcf()
         return ax
     
